/*
* XPathSaxonNS.java
* Copyright (C) 2004 Frederic Laurent
* http://www.opikanoba.org
*
* This file is part of the XPath Article.
*
* Lantern is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Lantern is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the Xpath article; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

/**
 * Test XPath avec la librairie Saxon.
 * Ce programme est volontairement trs simple. Le but n'est pas
 * de tester tous les cas possibles et les cas d'erreur mais de 
 * donner rapidement un aperu des objets  utiliser.
 * 
 * @author Frderic Laurent
 */
 
package test;


import java.net.URL;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.sax.SAXSource;

import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.xpath.StandaloneContext;
import net.sf.saxon.xpath.XPathEvaluator;
import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.xpath.XPathExpression;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;


/*
 * Classe charge de collecter les espaces de noms
 * Le prfixe des espaces de noms par dfaut est nomm par la
 * constante DEFAULT_NS_PREFIX + un numro qui augmente  chaque
 * fois qu'un nouveau espace de noms par dfaut est trouv.
 * 
 */

class NamespacesHandler extends DefaultHandler {
	
	private int defaultNSCount;
	private StandaloneContext context;
	
	public void startPrefixMapping(String prefix, String uri)
    throws SAXException {
		String pref=null;
		
		if ("".equals(prefix)){
			// espace de noms par dfaut
			pref=XPathSaxonNS.DEFAULT_NS_PREFIX+this.defaultNSCount;
			this.defaultNSCount++;
		} else {
			pref=prefix;
		}
		System.out.println("Namespaces\t" + pref + "\t" + uri);
		this.context.declareNamespace(pref,uri);
	}
	/**
	 * @param context contexte contenant en autres la dfinition des
	 * prfixes et URI des espaces de noms
	 * 
	 */
	public NamespacesHandler(StandaloneContext context) {
		this.defaultNSCount=1;
		this.context=context;
	}

}

public class XPathSaxonNS {
	// Document DOM sur lequel les requetes XPAth sont effectues
	private Document doc = null;
	// Evaluateur d'expressions XPath
	private XPathEvaluator xpathEval;
	public static final String DEFAULT_NS_PREFIX = "dns";
	
	
	/**
	 * Construction de la classe de test
	 */
	public XPathSaxonNS() {
		this.xpathEval=null;
		
		
	}
	
	/**
	 * Lecture d'un document XML, un fichier local ou distant (http)
	 * 
	 * @param xmlbuf fichier XML
	 */
	public void loadDocument(String xmlbuf) {
		InputSource is;
		try {
			if (xmlbuf.startsWith("http://") || xmlbuf.startsWith("file:")){
				is = new InputSource(new URL(xmlbuf).toString());	
			} else {
				is = new InputSource(xmlbuf);
			}
			
			// creation d'un contexte pour le support des espaces de noms
			StandaloneContext ctx = new StandaloneContext();
			// creation d'un parseur SAX pour trouver les espaces de noms
			SAXParserFactory saxFactory = SAXParserFactory.newInstance();
			XMLReader parser = saxFactory.newSAXParser().getXMLReader();
			// positionnement du handler charger de collecter les espaces de noms
	        parser.setContentHandler(new NamespacesHandler(ctx));
	        parser.setFeature("http://xml.org/sax/features/namespaces", true);
	        parser.parse(is);
			
	        // affectation du flux XML
			this.xpathEval = new XPathEvaluator(new SAXSource(is));
			// affectation du contexte contenant les espaces de noms
			this.xpathEval.setStaticContext(ctx);
			
		} catch (Exception e) {
			// si une exception quelconque survient, elle est trace
			// Attention, le traitement est volontairement simple dans un souci 
			// d'allgement du code ! un traitement correct serait beaucoup plus
			// dtaill
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
    }

	
	/**
	 * Evaluation d'une expression XPath
	 * - cration d'une valuateur d'expression  partir du document DOM
	 * - parcours de l'ensemble des noeuds rsutat et affichage de chacun
	 * @param xpath expression  valuer
	 */
	public void evalXpath(String xpath) {
		// cration de l'expression  valuer
		XPathExpression expr;
		try {
			expr = this.xpathEval.createExpression(xpath);
			// valuation de l'expression XPath
			List results = expr.evaluate();
			
			// parcours des noeuds correspondant  l'expression XPath
			// et affichage de chacun
			for (Iterator it = results.iterator(); it.hasNext();) {
				NodeInfo res = (NodeInfo)it.next();
				System.out.println(res.getStringValue());
			}
		} catch (XPathException e) {
			// si une exception quelconque survient, elle est trace
			// Attention, le traitement est volontairement simple dans un souci 
			// d'allgement du code ! un traitement correct serait beaucoup plus
			// dtaill
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	/**
	 * @param args arguments de la ligne de commande
	 */
	public static void main(String[] args) {
	
		if (args.length<2) {
			System.err.println("usage: java test.XPathSaxonNS xml xpath");
			System.err.println("   - xml   document XML");
			System.err.println("   - xpath expression XPath");
			System.exit(1);
		}
				
		final String xml= args[0];
		final String xpath = args[1];

		// initialisation de la classe de test
		XPathSaxonNS test = new XPathSaxonNS();
		// lecture du document
		test.loadDocument(xml);
		// evaluation de l'expression XPath
		test.evalXpath(xpath);
	}
}